##############################################
#
# text_cell
# Package | Text Cell Module.
#
# Daniel Bakas Amuchastegui
# February, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import text_cell
from .text_cell import *


##############################################
#
# genealogical_tree
# Module | genealogical_tree Module.
#
# Daniel Bakas Amuchastegui
# March 16, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .network import Network

class GenealogicalTree(Network):
    pass


##############################################
#
# Model
# Module | Module Model.
#
# Daniel Bakas Amuchastegui
# March 20, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .node import Node

class Model(Node):
    def __init__(self, id = None, dictionary = None):
        super().__init__(id, dictionary)



##############################################
#
# graph
# Module | Graph Module.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ...core.model import Model

class Graph(Model):
    pass


##############################################
#
# user 
# Package | User Package.
#
# Author: Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import admin
from . import author
from . import editor
from . import guest
from . import user

from .admin import *
from .author import *
from .editor import *
from .guest import *
from .user import *


##############################################
#
# text_cell
# Module | Text Cell Module.
#
# Daniel Bakas Amuchastegui
# February 20, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ..cell import Cell

class TextCell(Cell):
    pass


##############################################
#
# dvm_extract_cell
# Package | Data Visualization Model Extract Cell Package.
#
# Daniel Bakas Amuchastegui
# February, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import dvm_extract_cell
from .dvm_extract_cell import *


##############################################
#
# table
# Module | Displays an indexed, structured, table representating a given Resource's content.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ..core.model import Model

class Table(Model):
    pass


##############################################
#
# image_cell
# Package | Image Cell Module.
#
# Daniel Bakas Amuchastegui
# February, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import multimedia_cell
from .multimedia_cell import *


##############################################
#
# resource 
# Module | Resource Module.
#
# Author: Daniel Bakas Amuchastegui
# March 18, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ..core.node import Node
from ..namespace import RDF, RDFS, SMTK
from rdflib import Namespace, URIRef

class Resource(Node):
    _uri:URIRef

    def _get_uri(self):
        return self._uri
    def _set_uri(self, uri:URIRef):
        self._uri = uri
    uri = property(_get_uri, _set_uri)

    def _get_isDefinedBy(self):
        return self.get_values(str(RDFS.isDefinedBy))
    isDefinedBy = property(_get_isDefinedBy)

    def _get_label(self):
        return self.get_values(str(RDFS.label))
    label = property(_get_label)

    def _get_type(self):
        return self.get_values(str(RDF.type))
    type = property(_get_type)

    def _get_properties(self):
        for p in self.dictionary:
            yield p
    properties = property(_get_properties)

    def _get_subClassOf(self):
        return self.get_values(str(RDFS.subClassOf))
    subClassOf = property(_get_subClassOf)

    def _get_values(self):
        return self.get_values()
    values = property(_get_values)

    def add_properties(self, *properties:str):
        for p in properties:
            if p not in self.dictionary:
                self.dictionary[p] = set()

    def add_values(self, p:str, *objects):
        self.add_properties(p)
        for o in objects:
            self.dictionary[p].add(o)

    def delete_properties(self, *properties:str):
        if not properties:
            properties = self.dictionary
        for p in properties:
            if p in self.dictionary:
                del self.dictionary[p]

    def delete_values(self, p:str, *objects):
        for o in objects:
            if p in self.dictionary:
                self.dictionary[p].remove(o)

    def from_triples(self, triples:set):
        dictionary = dict()
        for (s,p,o) in triples:
            if str(p) not in dictionary:
                dictionary[str(p)] = set()
            dictionary[str(p)].add(o)
        return dictionary
            
    def get_values(self, *properties:str):
        if not properties:
            properties = self.dictionary
        for p in properties:
            if p in self.dictionary:
                for o in self.dictionary[p]:
                    yield o

    def to_triples(self):
        triples = set()
        for p in self.dictionary:
            for o in self.dictionary[p]:
                triples.add((self.uri, URIRef(p), o))
        return triples

    def update_property(self, old_p:str, new_p:str):
        self.dictionary[new_p] = self.dictionary[old_p]
        self.delete_properties(old_p)

    def update_values(self, p:str, old_o, *new_objects):
        self.delete_values(p, old_o)
        self.add_values(p, new_objects)

    def _generate_id(self, uri:URIRef = None):
        if uri and self.dictionary and str(RDFS.isDefinedBy) in self.dictionary:
            return str(uri).replace(list(self.dictionary[str(RDFS.isDefinedBy)])[0], '')
        return super()._generate_id()

    def _generate_uri(self, namespace:Namespace = None):
        result = URIRef((str(namespace or SMTK) + self.id or self._generate_id()))
        if self.dictionary and str(RDFS.isDefinedBy) in self.dictionary:
            result = URIRef(str(list(self.get_value(str(RDFS.isDefinedBy)))[0]) + self.id)
        return result

    def __eq__(self, other):
        return super().__eq__(other) and self.uri == other.uri

    def __hash__(self):
        return hash(self.uri)

    def __init__(self, id:str = None, dictionary:dict = None, triples:set = None, uri:URIRef = None):
        self.dictionary = dictionary or self.from_triples(triples)
        self.id = id or self._generate_id(uri)
        self.uri = uri

    def __iter__(self):
        return self.properties

    def __str__(self):
        result = str()
        result += '\n' + self.id + ':'
        for p in self:
            result += '\n - ' + str(p) + ':'
            for o in self.dictionary[p]:
                result += '\n  - ' + str(o)
        return result


##############################################
#
# author
# Module | Author Module.
#
# Daniel Bakas Amuchastegui
# March 18, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .user import User
from rdflib import URIRef

class Author(User): 
    def __init__(self, id:str = None, dictionary:dict = None, triples:set = None, uri:URIRef = None):
        super().__init__(id, dictionary, triples, uri)


##############################################
#
# chart
# Module | Chart Module.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .graph import Graph

class Chart(Graph):
    pass


##############################################
#
# av_reality
# Package | Augmented and Virtual Reality Package.
#
# Author: Daniel Bakas Amuchastegui
# September 6, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################



##############################################
#
# Graph
# Module | Module Graph.
#
# Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .resource import Resource
from ..core import graph
from ..core.node import Node
from ..namespace import RDF, RDFS, SMTK, XSD
from rdflib import Literal, URIRef

class Graph(graph.Graph):
    _triples:set

    def _get_triples(self):
        return self._triples
    def _set_triples(self, triples:set):
        self._triples = triples
    triples = property(_get_triples, _set_triples)

    def _get_objects(self):
        for s,p,o in self.triples:
            yield o
    objects = property(_get_objects)

    def _get_object_predicates(self):
        for s,p,o in self.triples:
            yield (o, p)
    object_predicates = property(_get_object_predicates)

    def _get_object_subjects(self):
        for s,p,o in self.triples:
            yield (o, s)
    object_subjects = property(_get_object_subjects)

    def _get_predicates(self):
        for s,p,o in self.triples:
            yield p
    predicates = property(_get_predicates)

    def _get_predicate_objects(self):
        for s,p,o in self.triples:
            yield (p, o)
    predicate_objects = property(_get_predicate_objects)

    def _get_predicate_subjects(self):
        for s,p,o in self.triples:
            yield (p, s)
    predicate_subjects = property(_get_predicate_subjects)

    def _get_subjects(self):
        for s,p,o in self.triples:
            yield s
    subjects = property(_get_subjects)

    def _get_subject_objects(self):
        for s,p,o in self.triples:
            yield (s, o)
    subject_objects = property(_get_subject_objects)

    def _get_subject_predicates(self):
        for s,p,o in self.triples:
            yield (s, p)
    subject_predicates = property(_get_subject_predicates)

    def _get_linked(self):
        triples = set()
        for s,p,o in self.triples:
            if isinstance(o, URIRef):
                triples.add((s,p,o))
        return Graph(self.id, triples = triples)
    linked = property(_get_linked)

    def _get_quantitative(self, linked:bool = False):
        triples = set()
        for s,p,o in self.triples:
            if (isinstance(o, Literal) and isinstance(o.value, (int, float)) and not isinstance(o.value, bool)):
                triples.add((s,p,o.value))
            if linked and isinstance(o, URIRef):
                triples.add((s,p,o.value))
        return Graph(self.id, triples = triples)
    quantitative = property(_get_quantitative)

    def _get_qualitative(self, linked:bool = False):
        triples = set()
        for s,p,o in self.triples:
            if isinstance(o, Literal) and (not isinstance(o.value, (int, float)) or isinstance(o.value, bool)):
                triples.add((s,p,o.value))
            if linked and isinstance(o, URIRef):
                triples.add((s,p,o.value))
        return Graph(self.id, triples = triples)
    qualitative = property(_get_qualitative)

    def add(self, *triples:tuple):
        for triple in triples:
            self.triples.add(triple)
    
    def remove(self, *triples:tuple):
        for triple in triples:
            self.triples.remove(triple)

    def to_dict(self):
        dictionary = dict()
        for (s,p,o) in self.triples:
            if str(s) not in dictionary:
                dictionary[str(s)] = dict()
            if str(p) not in dictionary[str(s)]:
                dictionary[str(s)][str(p)] = set()
            dictionary[str(s)][str(p)].add(o)
        return dictionary

    def to_resources(self):
        for s in self.to_dict():
            yield Resource(dictionary = self.to_dict()[s], uri = URIRef(s))

    def update(self, new_triple:tuple, old_triple:tuple):
        self.triples.remove(old_triple)
        self.triples.add(new_triple)

    def __init__(self, id:str = None, nodes:dict = None, triples:set = None, vertices = None):
        super().__init__(id, nodes, vertices)
        self.triples = triples

    def __iter__(self):
        for triple in self.triples:
            yield triple

    def __str__(self):
        result = str()
        for resource in self.to_resources():
            result += str(resource)
        return result


##############################################
#
# Node
# Module | Module Node.
#
# Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

import uuid

class Node(object):
    _dictionary:dict
    _id:str

    def _get_dictionary(self):
        return self._dictionary
    def _set_dictionary(self, dictionary:dict):
        self._dictionary = dictionary
    dictionary = property(_get_dictionary, _set_dictionary) 

    def _get_id(self):
        return self._id
    def _set_id(self, id:str):
        self._id = id
    id = property(_get_id, _set_id) 

    def _generate_id(self):
        result = '0'
        while result[0] not in {'a', 'b', 'c', 'd', 'e', 'f'}:
            result = uuid.uuid4().hex
            result = result[0:8] + '_' + result[8:12] + '_' + result[12:16] + '_' + result[16:20] + '_' + result[20:]
        return result

    def __eq__(self, other):
        return self.__class__ == other.__class__ and self.id == other.id and self.dictionary == other.dictionary

    def __hash__(self):
        return hash(self.id)

    def __init__(self, id:str = None, dictionary:dict = None):
        super().__init__()
        self.id = id or self._generate_id()
        self.dictionary = dictionary or dict()

    def __iter__(self):
        for key in self.dictionary:
            yield key


##############################################
#
# archive 
# Package | File System Package.
#
# Author: Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import graph
from . import resource
from . import user

from .graph import *
from .resource import *
from .user import *


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="semantyk",
    version="0.0.0.0",
    author="Daniel Bakas Amuchastegui",
    author_email="danielbakas@gmail.com",
    description="Semantyk | Ideas Wonder. | Visual Interactive Semantyk Intelligent Online Network.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/semantykcom/Semantyk",
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 1 - Planning",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3"
    ],
    python_requires='>=3.6',
)


##############################################
#
# Graph
# Module | Module Graph.
#
# Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from .node import Node

class Graph(Node):
    _vertices:set

    def _get_nodes(self):
        return self.dictionary
    def _set_nodes(self, nodes:dict):
        self.dictionary = nodes
    nodes = property(_get_nodes, _set_nodes)

    def _get_vertices(self):
        return self._vertices
    def _set_vertices(self, vertices:set):
        self._vertices = vertices
    vertices = property(_get_vertices, _set_vertices)

    def add_nodes(self, *nodes:Node):
        for node in nodes:
            if node.id not in self:
                self.nodes[node.id] = node.dictionary
            
    def delete_nodes(self, *nodes:Node):
        for node in nodes:
            if node.id in self:
                del self.nodes[node.id]

    def __eq__(self, other):
        return super().__eq__(other) and self.vertices == other.vertices

    def __hash__(self):
        return hash(self.id)

    def __init__(self, id:str, nodes:dict = None, vertices:set = None):
        super().__init__(id)
        self.nodes = nodes or dict()
        self.vertices = vertices

    def __iter__(self):
        for node in self.nodes:
            yield node 


##############################################
#
# network
# Module | network Module.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ..graph import Graph

class Network(Graph):
    pass


##############################################
#
# file
# Module | Structured and compact formats diplaying the content of a Resource, through the use of Cells.
#
# Daniel Bakas Amuchastegui
# September 3, 2019
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from ..core.model import Model

class File(Model):
    pass


##############################################
#
# core
# Package | Core Package.
#
# Author: Daniel Bakas Amuchastegui
# March 16, 2020
# 
# Copyright © Semantyk 2020. All rights reserved.
#############################################

from . import graph
from . import model
from . import node

from .graph import *
from .model import *
from .node import *